from django import template
from django.db.models import Q
from django.core.cache import cache
from django.core.urlresolvers import resolve, reverse, Resolver404
from farm.models import Genus, Breed, Animal
from django.core.cache import cache
from datetime import datetime
from django import template

register = template.Library()

class GetBreedsNode(template.Node):
    """
    Retrieves a list of breeds with animals in them and places it into the context
    """
    def __init__(self, varname):
        self.varname = varname

    def render(self, context):
        breeds = Breed.objects.all()
        context[self.varname] = breeds
        return ''

def get_breeds(parser, token):
    """
    Retrieves a list of breeds and places it into the context

    {% get_breeds as breeds %}
    """
    args = token.split_contents()
    argc = len(args)

    try:
        assert argc == 3 and args[1] == 'as'
    except AssertionError:
        raise template.TemplateSyntaxError('get_breeds syntax: {% get_breeds as varname %}')

    return GetBreedsNode(args[2])

class GetGenusNode(template.Node):
    """
    Retrieves a list of genus with animals in them and places it into the context
    """
    def __init__(self, all, ex_obj, varname):
        self.varname = varname
        self.ex_obj = ex_obj
        self.all = all

    def render(self, context):
        ex_obj = context.get(self.ex_obj, None)
        cache_key = 'genuses'
        genuses = cache.get(cache_key)
        if genuses is None:
            if all:
                genuses = Genus.objects.all()
            else:
                genus_ids=Animal.onthefarm_objects.values('primary_breed__genus').distinct()
                ids=[]
                for g in genus_ids:
                    ids.append(g['primary_breed__genus'])
                try:
                    genuses=Genus.objects.filter(id__in=ids).exclude(id=ex_obj.id)
                except:
                    genuses=Genus.objects.filter(id__in=ids)
            context[self.varname] = genuses
        return ''

def get_genuses(parser, token):
    """
    Retrieves a list of genus with animals on the farm in them and places it into the context

    {% get_genuses as genuses %}
    {% get_genuses all as genuses %}
    {% get_genuses as genuses exclude obj %}
    """
    args = token.split_contents()
    argc = len(args)

    try:
        assert argc == 3 or (argc == 4 and args[1].lower() == 'all') or (argc == 5 and args[-2].lower() == 'exclude')
    except AssertionError:
        raise template.TemplateSyntaxError('Invalid get_genuses syntax.')

    # determine what parameters to use
    ex_obj = varname = all = None
    if argc == 3: t, a, varname = args
    if argc == 4: t, all, a, varname = args
    elif argc == 5: t, a, varname, e, ex_obj = args

    if all: all = True

    return GetGenusNode(all=all, ex_obj=ex_obj, varname=varname)

class GetNamedAnmialsNode(template.Node):
    """
    Retrieves a list of animals on the farm with names and places it into the
    context.
    """
    def __init__(self, varname):
        self.varname = varname

    def render(self, context):
        cache_key = 'named_animals'
        animals = cache.get(cache_key)
        if animals is None:
            animals=Animal.onthefarm_objects.filter(name__isnull=False)
            context[self.varname] = animals
        return ''

def get_named_animals(parser, token):
    """
    Retrieves a list of named animals on the farm.

    {% get_named_animals as nanimals %}
    """
    args = token.split_contents()
    argc = len(args)

    try:
        assert argc == 3 and args[1] == 'as'
    except AssertionError:
        raise template.TemplateSyntaxError('get_named_animals syntax: {% get_named_animals as varname %}')

    return GetNamedAnimalsNode(args[2])

def child_table_for(obj, switch=''):
    """Provides a table of children for a given animal.
        
       Optionally responds to a switch for dam/sire.
    """

    # Get all the children for the specified object
    if switch == 'dam':
        children = obj.dam_of()
    elif switch == 'sire':
        children = obj.sire_of()
    else:
        children = obj.progeny()

    if len(children) == 0:
        # go no further
        return {}

    return {'children': children}

register.tag(get_genuses)
register.tag(get_breeds)
register.tag(get_named_animals)
register.inclusion_tag('farm/_child_table.html')(child_table_for) 




from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from onec_utils.utils import unique_slugify
from onec_utils.fields import CurrencyField
from onec_utils.models import USAddressPhoneMixin
from photologue.models import Photo
from markup_mixin.models import MarkupMixin
from django_extensions.db.models import TitleSlugDescriptionModel, TimeStampedModel
from notes.models import Note
from attributes.models import BaseAttribute, AttributeOption
from uuidfield import UUIDField

from farm.utils import get_fancy_time
from farm.managers import OnTheFarmManager 

class Farm(TitleSlugDescriptionModel, TimeStampedModel, USAddressPhoneMixin):
    """
    Farm model class.
    
    Info about a farm
    """
    contact=models.CharField(_('Contact'), max_length=200, blank=True, null=True)
    active=models.BooleanField(_('Active'), help_text='Is this your farm?', default=False)
        
    class Meta:
        verbose_name=_('Farm')
        verbose_name_plural=_('Farms')
  
    def __unicode__(self):
        return u'%s' % self.title

class Genus(TitleSlugDescriptionModel):
    """
    Genus model class.
    
    Keeps track of the various genus on a farm.
    """
    plural_name=models.CharField(_('Plural name'), help_text="Only use if adding an 's' does not work.", blank=True, null=True, max_length=200)
    technical_name=models.CharField(_('Technical title'), blank=True, null=True, max_length=200)
    
        
    class Meta:
        verbose_name=_('Genus')
        verbose_name_plural=_('Genus')
  
    def __unicode__(self):
        return u'%s' % self.title

    @models.permalink
    def get_absolute_url(self):
        return ('fm-genus-detail', None, {'slug': self.slug})

    @property
    def plural_title(self):
        if self.plural_name: return self.plural_name
        else: return self.title + 's'
        
class Breed(TitleSlugDescriptionModel):
    """
    Breed model class.
    
    Keeps track of the various breeds on a farm.
    """
    genus = models.ForeignKey(Genus)
        
    class Meta:
        verbose_name=_('Breed')
        verbose_name_plural=_('Breeds')
  
    def __unicode__(self):
        return u'%s %s' % (self.title, self.genus)

    @models.permalink
    def get_absolute_url(self):
        return ('fm-breed-detail', None, {'slug': self.slug, 'genus_slug': self.genus.slug})
        
class RegistrationBody(TitleSlugDescriptionModel):
    breed = models.ForeignKey(Breed)
    website = models.URLField(_('Website'), blank=True, null=True)

    class Meta:
        verbose_name=_('Registration body')
        verbose_name_plural=_('Registration bodies')
  
    def __unicode__(self):
        return u'%s' % self.name

class SecondaryBreed(models.Model):
    breed = models.ForeignKey(Breed)
    percentage = models.IntegerField(_('Percentage'), max_length=2)
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey("content_type", "object_id")
    
    class Meta:
        verbose_name=_('Secondary breed')
        verbose_name_plural=_('Secondary breeds')
    
    def __unicode__(self):
        return u'%s (%s)' % (self.breed, self.percentage )

class Animal(MarkupMixin, TimeStampedModel):
    """
    Animal model class.
    
    Keeps track of individual animals on a farm
    """
    SEX_CHOICES=[
            ('M', 'Male'),
            ('F', 'Female'),]

    uuid=UUIDField(auto=True, editable=True)
    name = models.CharField(_('Name'), blank=True, null=True, max_length=255)
    slug = models.SlugField(_('Slug'), blank=True)
    primary_breed = models.ForeignKey(Breed)
    dam = models.ForeignKey('self', related_name="dam_", blank=True, null=True)
    sire = models.ForeignKey('self', related_name="sire_", blank=True, null=True)
    breeder_farm=models.ForeignKey(Farm, related_name='breeder_farm', blank=True, null=True)
    alt_breeder=models.CharField(_('Other breeder'), blank=True, null=True, max_length=255)
    owner_farm= models.ForeignKey(Farm, related_name='owner_farm', blank=True, null=True)
    alt_owner=models.CharField(_('Other owner'), blank=True, null=True, max_length=255)
    birthday = models.DateField(_('Birthday'), blank=True, null=True)
    birthtime = models.TimeField(_('Birthtime'), blank=True, null=True)
    deathday = models.DateField(_('Deathday'), blank=True, null=True)
    description = models.TextField(_('Description'), blank=True, null=True)
    rendered_description = models.TextField(_('Rendered description'), blank=True, null=True, editable=False)
    photos=models.ManyToManyField(Photo, blank=True, null=True)
    sex=models.CharField(_('Sex'), choices=SEX_CHOICES, default='f', max_length=1)

    notes=generic.GenericRelation(Note)
    secondary_breeds=generic.GenericRelation('SecondaryBreed')
        
    objects = models.Manager()
    onthefarm_objects = OnTheFarmManager()

    class Meta:
        verbose_name=_('Animal')
        verbose_name_plural=_('Animals')

    class MarkupOptions:
        source_field = 'description'
        rendered_field = 'rendered_description'

    def save(self, *args, **kwargs):
        super(Animal, self).save(*args, **kwargs)

        if not self.name:
            self.slug = self.uuid
        else:
            unique_slugify(self, self.name)

        super(Animal, self).save()
        
    def __unicode__(self):
        if self.name:
            return u'%s a %s' % (self.name, self.primary_breed )
        elif self.dam:
            return u'%s from %s (ID: %s)' % (self.sex, self.dam.name, self.uuid[:10])
        else:
            return u'ID: %s - %s' % (self.uuid[:10], self.primary_breed)

    def __init__(self, *args, **kwargs):
        super (Animal, self).__init__(*args, **kwargs)
        self._mixed_breed = None
        self._registrations = None
        self._births = []
        self._litters = {}
        self._age = None

    @property
    def display_name(self):
        if self.name:
            return self.name
        elif self.dam:
            return u'Unnamed child of %s' %(self.dam.name)
        else:
            return u'Unnamed %s' %(self.primary_breed)

    @property
    def registrations(self):
        if not self._registrations:
            self._registrations = self.registration_set.all()
        return self._registrations

    @property
    def age(self):
        if not self._age:
            if self.birthday:
                if self.deathday: DELTA=self.deathday
                else: DELTA=datetime.now()
                self._age = get_fancy_time(relativedelta(DELTA, self.birthday), True)
        return self._age

    @property
    def breed(self):
        if self.mixed_breed():
            self._breed = 'Mixed ' + self.primary_breed.__unicode__()
        else:
            self._breed = self.primary_breed.__unicode__()
        return self._breed

    def mixed_breed(self):
        if not self._mixed_breed:
            if self.secondary_breeds.all():
                return True
            else:
                return False

    @property
    def location(self):
        if self.current_farm: return self.current_farm
        else: return self.alt_location

    @property
    def origin(self):
        if self.origin_farm: return self.origin_farm
        else: return self.alt_origin

    def sire_of(self):
        return Animal.onthefarm_objects.filter(sire=self)

    def dam_of(self):
        return Animal.onthefarm_objects.filter(dam=self)

    def progeny(self):
        return Animal.onthefarm_objects.filter(models.Q(sire=self)|models.Q(dam=self))

    def birthed_progeny(self):
        return Animal.objects.filter(models.Q(sire=self)|models.Q(dam=self))

    def lost_progeny(self):
        return Animal.objects.filter(models.Q(sire=self)|models.Q(dam=self)).filter(owner_farm__active=True).filter(deathday__isnull=False)

    def births(self):
    # Set the threshold for when births are consider contiguous
    # Some animals can go hours between births, leading into the next day easily
        delta = timedelta(days=1)
        if not self._births:
            for p in self.progeny():
                if not p.birthday in self._births:
                    recorded=False
                    for b in self._births:
                        if b+delta >  p.birthday > b-delta:
                            recorded=True
                    if not recorded:
                        self._births.append(p.birthday)
        return self._births

    def litters(self):
    # Set the threshold for when births are consider contiguous
    # Some animals can go hours between births, leading into the next day easily
        delta = timedelta(days=1)
        if not self._litters:
            for b in self.births():
                p_collection=[]
                for p in self.birthed_progeny():
                    if b+delta > p.birthday > b-delta:
                        p_collection.append(p)
                self._litters[b]=p_collection
        return self._litters


    @models.permalink
    def get_absolute_url(self):
        return ('fm-animal-detail', None, {'slug': self.slug, 'breed_slug': self.primary_breed.slug, 'genus_slug': self.primary_breed.genus.slug})


class AnimalAttributeOption(AttributeOption):

    class Meta:
        verbose_name = _('Animal attribute options')
        verbose_name_plural = _('Animal attribute options')

class AnimalAttribute(BaseAttribute):
    option = models.ForeignKey(AnimalAttributeOption)
    animal = models.ForeignKey(Animal)

    class Meta:
        verbose_name = _('Animal attribute')
        verbose_name_plural = _('Animal attributes')

    def __unicode__(self):
        return u'%s attribute of %s' %(self.option, self.animal.display_name)

class AnimalRegistration(models.Model):
    animal = models.ForeignKey(Animal)
    body = models.ForeignKey(RegistrationBody)
    date = models.DateField(_('Registration date'), blank=True, null=True)
    reg_id = models.CharField(_('Registration ID'), blank=True, null=True, max_length=255)

    class Meta:
        verbose_name=_('Animal registration')
        verbose_name_plural=_('Animals registrations')
  
    def __unicode__(self):
        return u'Registration of %s at %s' % (self.animal, self.body)
    
class ProductType(TitleSlugDescriptionModel):
    """
    ProductType model class.
    
    Keeps track of the various product types on a farm, such as produce, meat, soap, preserves etc...
    """
        
    class Meta:
        verbose_name=_('Product type')
        verbose_name_plural=_('Product types')
  
    def __unicode__(self):
        return u'%s' % self.title

    @models.permalink
    def get_absolute_url(self):
        return ('fm-product-type-detail', None, {'slug': self.slug})
        
class ProductAttributeOption(AttributeOption):

    class Meta:
        verbose_name = _('Product attribute options')
        verbose_name_plural = _('Product attribute options')

class ProductAttribute(BaseAttribute):
    product = models.ForeignKey('Product')

    class Meta:
        verbose_name = _('Product attribute')
        verbose_name_plural = _('Product attributes')

class Product(TitleSlugDescriptionModel, TimeStampedModel):
    type = models.ForeignKey(ProductType)
    photos=models.ManyToManyField(Photo, blank=True, null=True)
    price=CurrencyField(_('Price'), blank=True, null=True, decimal_places=2, max_digits=5)
    unit=models.CharField(_('Unit'), blank=True, null=True, max_length=100)
    verbose_price=models.CharField(_('Verbose price'), blank=True, null=True, max_length=255)

    class Meta:
        verbose_name=_('Product')
        verbose_name_plural=_('Products')
  
    def __unicode__(self):
        return u'%s' % self.title
        
    @models.permalink
    def get_absolute_url(self):
        return ('fm-product-detail', None, {'slug': self.slug, 'type_slug': self.type.slug})

class Building(MarkupMixin, TitleSlugDescriptionModel, TimeStampedModel):
    farm=models.ForeignKey(Farm)
    built=models.DateField(_('Built'), blank=True, null=True)
    photos=models.ManyToManyField(Photo, blank=True, null=True)
    rendered_description=models.TextField(_('Rendered description'), blank=True, null=True, editable=False)

    class Meta:
        verbose_name=_('Building')
        verbose_name_plural=_('Buildings')

    class MarkupOptions:
        source_field = 'description'
        rendered_field = 'rendered_description'
  
    def __unicode__(self):
        return u'%s' % self.title

    def __init__(self, *args, **kwargs):
        super (Building, self).__init__(*args, **kwargs)
        self._age = None
        
    @property
    def age(self):
        if not self._age:
            if self.built:
                DELTA=datetime.now()
                self._age = get_fancy_time(relativedelta(DELTA, self.built), True)
            else: self._age = "Unknown"
        return self._age

    @models.permalink
    def get_absolute_url(self):
        return ('fm-building-detail', None, {'slug': self.slug})

class BuildingAttributeOption(AttributeOption):

    class Meta:
        verbose_name = _('Building attribute options')
        verbose_name_plural = _('Building attribute options')

class BuildingAttribute(BaseAttribute):
    option = models.ForeignKey(BuildingAttributeOption)
    building = models.ForeignKey(Building)

    class Meta:
        verbose_name = _('Building attribute')
        verbose_name_plural = _('Building attributes')

    def __unicode__(self):
        return u'%s attribute of %s' %(self.option, self.building.title)

class BuildingSpace(MarkupMixin, TitleSlugDescriptionModel):
    """
    Bulding Space model class.

    Keeps track of the various spaces in a bulding (rooms, lofts, corners
    etc...)
    """
    building=models.ForeignKey(Building)
    photos = models.ManyToManyField(Photo, blank=True, null=True)
    rendered_description=models.TextField(_('Rendered description'), blank=True, null=True)

    class Meta:
        verbose_name=_('Building space')
        verbose_name_plural=_('Building spaces')

    class MarkupOptions:
        source_field = 'description'
        rendered_field = 'rendered_description'
  
    def __unicode__(self):
        return u'%s' % self.title

    def __init__(self, *args, **kwargs):
        super (BuildingSpace, self).__init__(*args, **kwargs)

class FieldType(TitleSlugDescriptionModel):
    """
    Field Type model class.
    
    Keeps track of the various field types on a farm, such as pasture, garden,
    grain, etc...
    """
        
    class Meta:
        verbose_name=_('Field type')
        verbose_name_plural=_('Field types')
  
    def __unicode__(self):
        return u'%s' % self.title

    @models.permalink
    def get_absolute_url(self):
        return ('fm-field-type-detail', None, {'slug': self.slug})

class Field(MarkupMixin, TitleSlugDescriptionModel, TimeStampedModel):
    farm=models.ForeignKey(Farm)
    type=models.ForeignKey(FieldType, blank=True, null=True)
    photos=models.ManyToManyField(Photo, blank=True, null=True)
    rendered_description=models.TextField(_('Rendered description'), blank=True, null=True, editable=False)

    class Meta:
        verbose_name=_('Field')
        verbose_name_plural=_('Fields')

    class MarkupOptions:
        source_field = 'description'
        rendered_field = 'rendered_description'
  
    def __unicode__(self):
        return u'%s' % self.title

    def __init__(self, *args, **kwargs):
        super (Field, self).__init__(*args, **kwargs)
        
    @models.permalink
    def get_absolute_url(self):
        return ('fm-field-detail', None, {'slug': self.slug})

class FieldAttributeOption(AttributeOption):

    class Meta:
        verbose_name = _('Field attribute options')
        verbose_name_plural = _('Field attribute options')

class FieldAttribute(BaseAttribute):
    option = models.ForeignKey(FieldAttributeOption)
    field = models.ForeignKey(Field)

    class Meta:
        verbose_name = _('Field attribute')
        verbose_name_plural = _('Field attributes')

    def __unicode__(self):
        return u'%s attribute of %s' %(self.option, self.field.title)



from datetime import datetime
from dateutil.relativedelta import relativedelta

def get_fancy_time(rdelta, display_full_version = False):
    """Returns a user friendly date format
    d: some datetime instace in the past
    display_second_unit: True/False
    """
    #some helpers lambda's
    plural = lambda x: 's' if x > 1 else ''
    singular = lambda x: x[:-1]
    #convert pluran (years) --> to singular (year)
    display_unit = lambda unit, name: '%s %s%s'%(unit, name, plural(unit)) if unit > 0 else ''

    #time units we are interested in descending order of significance
    tm_units = ['years', 'months', 'days', 'hours', 'minutes', 'seconds']

    for idx, tm_unit in enumerate(tm_units):
        first_unit_val = getattr(rdelta, tm_unit)
        if first_unit_val > 0:
            primary_unit = display_unit(first_unit_val, singular(tm_unit))
            if display_full_version and idx < len(tm_units)-1:
                next_unit = tm_units[idx + 1]
                second_unit_val = getattr(rdelta, next_unit)
                if second_unit_val > 0:
                    secondary_unit = display_unit(second_unit_val, singular(next_unit))
                    return primary_unit + ', ' + secondary_unit
            return primary_unit
    return None

if __name__ == "__main__":
    data_sets = [datetime.utcnow() + relativedelta(seconds=-3),
                        datetime.utcnow() + relativedelta(minutes=-2, seconds=-6),
                        datetime.utcnow() + relativedelta(minutes=-65, seconds=-50),
                        datetime.utcnow() + relativedelta(days=-2, minutes=-2),
                        datetime.utcnow() + relativedelta(months=-2, days=-45),
                        datetime.utcnow() + relativedelta(months=-54)]

    for x in data_sets:
        print '%s' % (get_fancy_time(x, display_full_version = True))


ANIMAL_FEMALE_NAMES={
        'pig':     'sow',
        'chicken': 'hen',
        'goat':    'doe',
        'dog':     'bitch',
        'sheep':   'ewe'}




import datetime
from django.views.generic import DetailView, ListView

from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import get_list_or_404, render_to_response, get_object_or_404
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.conf import settings

from farm.models import Animal, Breed, Product
from notes.forms import BriefNoteForm

class BreedDetailView(DetailView):
    model = Breed

    def get_queryset(self, *args, **kwargs):
        return Breed.objects.filter(genus__slug=self.kwargs['genus_slug'])

class ProductDetailView(DetailView):
    model = Product

    def get_queryset(self, *args, **kwargs):
        return Product.objects.filter(type__slug=self.kwargs['type_slug'])

class AnimalDetailView(DetailView):
    model = Animal

    def get_queryset(self, *args, **kwargs):
        return Animal.objects.filter(primary_breed__genus__slug=self.kwargs['genus_slug'], primary_breed__slug=self.kwargs['breed_slug'])

    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()

        pk = self.kwargs.get('pk', None)
        slug_or_uuid = self.kwargs.get('slug', None)
        if pk is not None:
            queryset = queryset.filter(pk=pk)

        # Next, try looking up by slug.
        elif slug_or_uuid is not None:
            try:
                obj = queryset.get(uuid=slug_or_uuid)
            except:
                try:
                    obj = queryset.get(slug=slug_or_uuid)
                except ObjectDoesNotExist:
                    raise Http404(_(u"No %(verbose_name)s found matching the query") %
                                   {'verbose_name': queryset.model._meta.verbose_name})

        # If none of those are defined, it's an error.
        else:
            raise AttributeError(u"Generic detail view %s must be called with "
                                 u"either an object pk or a slug."
                                 % self.__class__.__name__)

        return obj

    def get_context_data(self, **kwargs):
        context = super(AnimalDetailView, self).get_context_data(**kwargs)
        context['note_form'] = BriefNoteForm()
        return context



import logging
from django.forms import models, ValidationError
from django.contrib import admin
from django.contrib.contenttypes import generic

from farm.models import Farm, Genus, Breed, Animal, Product, ProductType, Note, SecondaryBreed, AnimalAttribute, AnimalAttributeOption, ProductAttribute, ProductAttributeOption, Building, BuildingSpace, Field, FieldType, BuildingAttribute, FieldAttribute, BuildingAttributeOption, FieldAttributeOption
from notes.admin import NoteInline
from attributes.admin import clean_attribute_value

class AnimalAttributeInlineForm(models.ModelForm):

    def clean_value(self):
        print 'Farm admin.py: cleaning values and have data: %s' % self.cleaned_data
        return clean_attribute_value(self.cleaned_data, self.cleaned_data['animal'])

class AnimalAttributeInline(admin.TabularInline):
    model = AnimalAttribute
    #form = AnimalAttributeInlineForm

class ProductAttributeInline(admin.TabularInline):
    model = ProductAttribute

class SecondaryBreedInline(generic.GenericTabularInline):
    model = SecondaryBreed

class AnimalAdmin(admin.ModelAdmin):
    inlines = [ AnimalAttributeInline, SecondaryBreedInline, ]

class ProductAdmin(admin.ModelAdmin):
    inlines = [ ProductAttributeInline, NoteInline, ]

admin.site.register(Animal, AnimalAdmin)
admin.site.register(Farm)
admin.site.register(Genus)
admin.site.register(Breed)
admin.site.register(Product, ProductAdmin)
admin.site.register(ProductType)
admin.site.register(AnimalAttribute)
admin.site.register(AnimalAttributeOption)
admin.site.register(ProductAttributeOption)
admin.site.register(Building)
admin.site.register(BuildingSpace)
admin.site.register(Field)
admin.site.register(FieldType)
admin.site.register(BuildingAttribute)
admin.site.register(FieldAttribute)
admin.site.register(BuildingAttributeOption)
admin.site.register(FieldAttributeOption)



from django.db.models import Manager
from datetime import datetime

class OnTheFarmManager(Manager):
    def get_query_set(self):
        return super(OnTheFarmManager, self).get_query_set().filter(owner_farm__active=True).filter(deathday__isnull=True)




from django.conf import settings
from django.conf.urls.defaults import *
from django.views.generic import DetailView, ListView
from farm.views import BreedDetailView, AnimalDetailView, ProductDetailView
from farm.models import Animal, Genus, Breed, Product, ProductType, RegistrationBody, AnimalRegistration, Building, Field


# custom views vendors
urlpatterns = patterns('',
    url(r'^animals/$', view=ListView.as_view(model=Genus), name="fm-genus-list"),
	url(r'^animals/(?P<slug>[-\w]+)/$', view=DetailView.as_view(model=Genus), name="fm-genus-detail"),
	url(r'^animals/(?P<slug>[-\w]+)/$', view=DetailView.as_view(model=Genus), name="fm-genus-detail"),
	url(r'^animals/(?P<genus_slug>[-\w]+)/(?P<slug>[-\w]+)/$', view=BreedDetailView.as_view(), name="fm-breed-detail"),
	url(r'^animals/(?P<genus_slug>[-\w]+)/(?P<breed_slug>[-\w]+)/(?P<slug>[-\w]+)/$', view=AnimalDetailView.as_view(), name="fm-animal-detail"),

    url(r'^products/$', view=ListView.as_view(model=Product), name="fm-product-list"),
    url(r'^products/(?P<slug>[-\w]+)/$', view=DetailView.as_view(model=ProductType), name="fm-product-type-detail"),
    url(r'^products/(?P<type_slug>[-\w]+)/(?P<slug>[-\w]+)/$', view=ProductDetailView.as_view(), name="fm-product-detail"),

    url(r'^buildings/$', view=ListView.as_view(model=Building), name="fm-building-list"),
    url(r'^buildings/(?P<slug>[-\w]+)/$', view=DetailView.as_view(model=Building), name="fm-building-detail"),
    url(r'^buildings/(?P<building-slug>[-\w]+)/(?P<slug>[-\w]+)/$', view=DetailView.as_view(model=BuildingSpace), name="fm-building-space-detail"),

    url(r'^fields/$', view=ListView.as_view(model=Field), name="fm-field-list"),
    url(r'^fields/(?P<slug>[-\w]+)/$', view=DetailView.as_view(model=Field), name="fm-field-detail"),
)
